/*** INI parser, Cristiaan Kop								 	 updated 05-01-2017 ***/
/*** ============================================================================== ***/
/*** NAMESPACE: Crealysm::COMMON													***/
/*** header file for the CIniParser class interface					     		    ***/

#ifndef CINI_PARSER_H
#define CINI_PARSER_H

#include <string>
#include <fstream>
#include <vector>


namespace Crealysm { namespace COMMON
{

/**************************************************************************************/
/***								CIniParser class								***/
/*** class for reading/updating INI files and retrieving data from them: parsing 	***/
/**************************************************************************************/

class CIniParser
{
public:
	CIniParser();
	~CIniParser();

	/***** START AND STOP READING *****/
	bool StartReading(const std::string &pFilename);
	bool StopReading();

	/***** SET THE CURRENT FILENAME FOR WRITING *****/
	void SetWriteFile(const std::string &pFilename);

	// Reading: moves filestream to a requested or the next block
	bool GetNextBlockType(unsigned int &pNrAttributes, std::string &pBlockName);
	bool GotoBlockByName(const std::string &pBlockName, const unsigned int pBlockId, unsigned int &pNrAttributes);
		
	// Reading: retrieve next attribute's name
	std::string GetNextAttributeName();

	// Reading: count blocks with a given name
	bool CountBlocks(const std::string &pBlockName, unsigned int &pNrBlocks);
	int GetBlockIndexByVal(const std::string &pBlockName, const std::string &pAttributeName, const unsigned int pVal);
	int GetBlockIndexByVal(const std::string &pBlockName, const std::string &pAttributeName, const std::string &pVal);

	// Reading: retrieve attribute's value
	int				GetInt();
	bool			GetBool();
	std::string		GetString();

	/***** WRITING *****/
	bool UpdateInt(const std::string &pBlockName, const unsigned int pBlockId, const std::string &pAttrib, const int pIntVal);
	bool UpdateBool(const std::string &pBlockname, const unsigned int pBlockId, const std::string &pAttrib, const bool pBoolVal);
	bool UpdateString(const std::string &pBlockName, const unsigned int pBlockId, const std::string &pAttrib, const std::string pStringVal);

private:
	std::ifstream mReadIni;
	
	std::ofstream mWriteIni;
	std::vector<std::string> mTempReadLines;
	std::string mWriteFile;

	// Helper functions
	bool GetBlockName(const std::string &pLine, std::string &pBlockName)	const;

	bool IsBlockLine(const std::string &pLine)			const;
	bool IsCommentLine(const std::string &pLine)		const; 
	bool IsEmptyLine(const std::string &pLine)			const;

	int CountAttributes();

	/***** WRITING *****/
	bool PrepareUpdate(const std::string &pBlockName, const unsigned int pBlockId, const std::string &pAttrib, unsigned int *pStopLine);
	bool FinishUpdate(const unsigned int pContinueLine);
};

}
}

#endif